@@ -5,7 +5,9 @@
 //! internal power supply.
 
 use std::collections::HashMap;
+use std::fs::{read_dir, read_to_string};
 use std::path::{Path, PathBuf};
+use std::sync::{Arc, Mutex};
 use std::thread;
 use std::time::{Duration, Instant};
 
@@ -320,14 +322,10 @@ pub struct ApcUpsDevice {
 
 impl ApcUpsDevice {
     pub fn from_device(device: &str, allow_missing: bool) -> Result<ApcUpsDevice> {
-        let mut device_addr = device;
-        if !device_addr.contains(':') {
-            device_addr = "localhost:3551";
-        }
         Ok(ApcUpsDevice {
-            con: ApcAccess::new(device_addr, 1).block_error(
+            con: ApcAccess::new(device, 1).block_error(
                 "battery",
-                &format!("Could not create a apcaccess connection to {}", device_addr),
+                &format!("Could not create a apcaccess connection to {}", device),
             )?,
             status: None,
             allow_missing,
@@ -451,7 +449,8 @@ impl BatteryDevice for ApcUpsDevice {
 }
 
 pub struct UpowerDevice {
-    device_path: String,
+    device: String,
+    device_path: Arc<Mutex<Option<String>>>,
     con: dbus::ffidisp::Connection,
     allow_missing: bool,
 }
@@ -461,63 +460,77 @@ impl UpowerDevice {
     /// the path `"/org/freedesktop/UPower/devices/<device>"`. Raises an error
     /// if D-Bus does not respond.
     pub fn from_device(device: &str, allow_missing: bool) -> Result<Self> {
-        let device_path = format!("/org/freedesktop/UPower/devices/{}", device);
-        let con = dbus::ffidisp::Connection::get_private(dbus::ffidisp::BusType::System)
+        let con = dbus::ffidisp::Connection::new_system()
             .block_error("battery", "Failed to establish D-Bus connection.")?;
 
-        Ok(UpowerDevice {
-            device_path,
-            con,
-            allow_missing,
-        })
+        let device_path = UpowerDevice::get_device_path(device, &con)?;
+
+        if device_path.is_some() || allow_missing {
+            Ok(UpowerDevice {
+                device: device.to_string(),
+                device_path: Arc::new(Mutex::new(device_path)),
+                con,
+                allow_missing,
+            })
+        } else {
+            Err(BlockError(
+                "battery".to_string(),
+                "UPower device could not be found.".to_string(),
+            ))
+        }
     }
 
     /// Monitor UPower property changes in a separate thread and send updates
     /// via the `update_request` channel.
     pub fn monitor(&self, id: usize, update_request: Sender<Task>) {
-        let path = self.device_path.clone();
+        let device = self.device.clone();
+        let device_path = self.device_path.clone();
         thread::Builder::new()
             .name("battery".into())
             .spawn(move || {
-                let con = dbus::ffidisp::Connection::get_private(dbus::ffidisp::BusType::System)
+                let con = dbus::ffidisp::Connection::new_system()
                     .expect("Failed to establish D-Bus connection.");
-                let properties_changed_rule = format!(
-                    "type='signal',\
-                 path='{}',\
+                let enumerate_con = dbus::ffidisp::Connection::new_system()
+                    .expect("Failed to establish D-Bus connection.");
+                let properties_changed_rule = "type='signal',\
                  interface='org.freedesktop.DBus.Properties',\
-                 member='PropertiesChanged'",
-                    path
-                );
+                 member='PropertiesChanged'";
+
                 let device_removed_rule = "type='signal',\
-                 interface='org.freedesktop.UPower',\
-                 member='DeviceRemoved'";
+                    interface='org.freedesktop.UPower',\
+                    member='DeviceRemoved'";
                 let device_added_rule = "type='signal',\
-                 interface='org.freedesktop.UPower',\
-                 member='DeviceAdded'";
+                    interface='org.freedesktop.UPower',\
+                    member='DeviceAdded'";
 
                 // First we're going to get an (irrelevant) NameAcquired event.
                 con.incoming(10_000).next();
 
-                con.add_match(&properties_changed_rule)
+                con.add_match(properties_changed_rule)
                     .expect("Failed to add D-Bus match rule.");
                 con.add_match(device_removed_rule)
                     .expect("Failed to add D-Bus match rule.");
                 con.add_match(device_added_rule)
                     .expect("Failed to add D-Bus match rule.");
 
-                let device_path = dbus::Path::from(&path);
                 loop {
                     if let Some(msg) = con.incoming(10_000).next() {
                         if let Some(interface) =
                             msg.interface().map(|interface| interface.to_string())
                         {
-                            if (interface == "org.freedesktop.UPower"
+                            let device_changed = interface == "org.freedesktop.UPower"
                                 && msg
                                     .get1::<dbus::Path>()
                                     .expect("Unable to get objectpath argument")
-                                    == device_path)
-                                || interface == "org.freedesktop.DBus.Properties"
-                            {
+                                    .starts_with("/org/freedesktop/UPower/devices/");
+
+                            // If has been added or removed update the device path
+                            if device_changed {
+                                *device_path.lock().unwrap() =
+                                    UpowerDevice::get_device_path(&device, &enumerate_con).unwrap();
+                            }
+
+                            if device_changed || interface == "org.freedesktop.DBus.Properties" {
                                 update_request
                                     .send(Task {
                                         id,
@@ -526,9 +539,6 @@ impl UpowerDevice {
                                     .unwrap();
                             }
                         }
-                        // Avoid update spam.
-                        // TODO: Is this necessary?
-                        thread::sleep(Duration::from_millis(1000))
                     }
                 }
             })
@@ -543,39 +553,59 @@ impl UpowerDevice {
         key: &str,
         fallback_value: T,
     ) -> Result<T> {
-        self.con
-            .with_path("org.freedesktop.UPower", &self.device_path, 1000)
-            .get::<T>("org.freedesktop.UPower.Device", key)
-            .or_else(|_| {
-                if self.allow_missing {
-                    return Ok(fallback_value);
-                }
-                Err(BlockError(
-                    "battery".into(),
-                    format!("Failed to read UPower {} property.", key),
-                ))
-            })
+        if let Some(device_path) = &*self.device_path.lock().unwrap() {
+            if let Ok(value) = self
+                .con
+                .with_path("org.freedesktop.UPower", device_path, 1000)
+                .get::<T>("org.freedesktop.UPower.Device", key)
+            {
+                return Ok(value);
+            }
+        }
+        if self.allow_missing {
+            return Ok(fallback_value);
+        }
+        Err(BlockError(
+            "battery".into(),
+            format!("Failed to read UPower {} property.", key),
+        ))
+    }
+
+    // Get device path. Raises exception if dbus communication fails.
+    // If the device doesn't exist an exception raised unless allow_missing == true
+    fn get_device_path(device: &str, con: &dbus::ffidisp::Connection) -> Result<Option<String>> {
+        if device == "DisplayDevice" {
+            Ok(Some(String::from(
+                "/org/freedesktop/UPower/devices/DisplayDevice",
+            )))
+        } else {
+            let msg = dbus::Message::new_method_call(
+                "org.freedesktop.UPower",
+                "/org/freedesktop/UPower",
+                "org.freedesktop.UPower",
+                "EnumerateDevices",
+            )
+            .block_error("battery", "Failed to create DBus message")?;
+
+            let dbus_reply = con
+                .send_with_reply_and_block(msg, 2000)
+                .block_error("battery", "Failed to retrieve DBus reply")?;
+
+            // EnumerateDevices returns one argument, which is an array of ObjectPaths (not dbus::tree:ObjectPath).
+            let mut paths: Array<dbus::Path, _> = dbus_reply
+                .get1()
+                .block_error("battery", "Failed to read DBus reply")?;
+
+            Ok(paths
+                .find(|entry| entry.ends_with(device))
+                .map(|path| path.to_string()))
+        }
     }
 }
 
 impl BatteryDevice for UpowerDevice {
     fn is_available(&self) -> bool {
-        if let Ok(msg) = dbus::Message::new_method_call(
-            "org.freedesktop.UPower",
-            "/org/freedesktop/UPower",
-            "org.freedesktop.UPower",
-            "EnumerateDevices",
-        ) {
-            if let Ok(dbus_reply) = self.con.send_with_reply_and_block(msg, 2000) {
-                // EnumerateDevices returns one argument, which is an array of ObjectPaths (not dbus::tree:ObjectPath).
-                if let Some(mut paths) = dbus_reply.get1::<Array<dbus::Path, _>>() {
-                    // Target device path
-                    let device_path = dbus::Path::from(&self.device_path);
-                    return paths.any(|entry| entry == device_path);
-                }
-            }
-        }
-        false
+        self.device_path.lock().unwrap().is_some()
     }
 
     fn refresh_device_info(&mut self) -> Result<()> {
@@ -676,7 +706,7 @@ pub struct BatteryConfig {
     pub interval: Duration,
 
     /// The internal power supply device in `/sys/class/power_supply/` to read from.
-    pub device: String,
+    pub device: Option<String>,
 
     /// Format string for displaying battery information.
     /// placeholders: {percentage}, {bar}, {time} and {power}
@@ -715,27 +745,11 @@ pub struct BatteryConfig {
     pub hide_missing: bool,
 }
 
-fn default_device() -> String {
-    let mut res = "BAT0".to_string();
-    let mut found = false;
-    if let Ok(dir) = std::fs::read_dir("/sys/class/power_supply") {
-        for entry in dir.flatten() {
-            if let Some(f) = entry.file_name().to_str() {
-                if f.starts_with("BAT") && (!found || f < res.as_str()) {
-                    found = true;
-                    res = f.to_string();
-                }
-            }
-        }
-    }
-    res
-}
-
 impl Default for BatteryConfig {
     fn default() -> Self {
         Self {
             interval: Duration::from_secs(10),
-            device: default_device(),
+            device: None,
             format: FormatTemplate::default(),
             full_format: FormatTemplate::default(),
             missing_format: FormatTemplate::default(),
@@ -760,19 +774,68 @@ impl ConfigBlock for Battery {
         shared_config: SharedConfig,
         update_request: Sender<Task>,
     ) -> Result<Self> {
+        let device_str = match block_config.device {
+            Some(d) => d,
+            None => match block_config.driver {
+                BatteryDriver::ApcAccess => "localhost:3551".to_string(),
+                BatteryDriver::Upower => "DisplayDevice".to_string(),
+                _ => {
+                    let sysfs_dir = read_dir("/sys/class/power_supply").block_error(
+                        "battery",
+                        "failed to read /sys/class/power_supply direcory",
+                    )?;
+                    let mut found_battery_devices = Vec::<String>::new();
+                    for entry in sysfs_dir {
+                        let dir = entry?;
+                        if read_to_string(dir.path().join("type"))
+                            .map(|t| t.trim() == "Battery")
+                            .unwrap_or(false)
+                        {
+                            found_battery_devices
+                                .push(dir.file_name().to_str().unwrap().to_string());
+                        }
+                    }
+
+                    // Better to default to the system battery, rather than possibly a keyboard or mouse battery.
+                    // System batteries usually start with BAT or CMB.
+                    // Otherwise, just grab the first one from the list.
+                    let chosen_device = if let Some(preferred_device) = found_battery_devices
+                        .iter()
+                        .find(|&s| s.starts_with("BAT") || s.starts_with("CMB"))
+                    {
+                        Some(preferred_device)
+                    } else {
+                        found_battery_devices.first()
+                    };
+
+                    match chosen_device {
+                        Some(d) => d.to_string(),
+                        None => {
+                            if block_config.allow_missing {
+                                // TODO: If the battery isn't actually BAT0, then even if it appears again we will never update
+                                // Need to implement device refresh
+                                "BAT0".to_string()
+                            } else {
+                                return Err(BlockError("battery".to_string(), "failed to determine default battery - please set your battery device in the configuration file".to_string()));
+                            }
+                        }
+                    }
+                }
+            },
+        };
+
         let device: Box<dyn BatteryDevice> = match block_config.driver {
             BatteryDriver::ApcAccess => Box::new(ApcUpsDevice::from_device(
-                &block_config.device,
+                &device_str,
                 block_config.allow_missing,
             )?),
             BatteryDriver::Upower => {
-                let out =
-                    UpowerDevice::from_device(&block_config.device, block_config.allow_missing)?;
+                let out = UpowerDevice::from_device(&device_str, block_config.allow_missing)?;
                 out.monitor(id, update_request);
                 Box::new(out)
             }
             BatteryDriver::Sysfs => Box::new(PowerSupplyDevice::from_device(
-                &block_config.device,
+                &device_str,
                 block_config.allow_missing,
             )?),
         };
